-- SPDX-License-Identifier: MIT
--
-- Ports saved volumes and other props from 0.4 to 0.5.
-- We ignore saved profile assignments for now; for a given setup, either the
-- default will just work or the user ends up setting card profiles manually
-- anyway.
--
-- Contains portions of src/scripts/policy-device-routes.lua
-- from WirePlumber 0.4.17 and src/scripts/device/state-routes.lua
-- from WirePlumber 0.5.3.

state = State("default-routes") or nil
state_t = state and state:load() or nil

-- True if at least 1 pre-0.5 prop is found.
has_old_props = false

function storeprop(t, p, v)
    -- In the 0.5 format, prop would be empty, and
    -- v is a JSON object with all the props.
    -- Catch this kind of input, as well as errors.
    if (not t) then
        return false
    end
    if (p == "") or (p:sub(1,1) ~= ":") then
        print(string.format('[  ign] "%s" == "%s"', p, v))
        return false
    end
    -- In the 0.4 format, prop would be non-empty
    -- and would correspond to a serialized object.
    local p = p:sub(2,-1)
    if p == 'mute' then
        t.mute = v and (v == "true") or false
    elseif p == 'volume' then
        t.volume = v and tonumber(v) or t.volume
    elseif p == 'channelVolumes' then
        t.channelVolumes = v and parseArray(v, tonumber) or t.channelVolumes
    elseif p == 'channelMap' then
        t.channelMap = v and parseArray(v) or t.channelMap
    elseif p == 'latencyOffsetNsec' then
        t.latencyOffsetNsec = v and math.tointeger(v) or t.latencyOffsetNsec
    elseif p == 'iec958Codecs' then
        t.iec958Codecs = v and parseArray(v) or t.iec958Codecs
    else
        return false
    end
    has_old_props = true
    return true
end

-- Simple deserializer "foo;bar;" -> {"foo", "bar"} from 0.4.
function parseArray(str, convert_value)
    local array = {}
    local val = ""
    local escaped = false
    for i = 1, #str do
        local c = str:sub(i,i)
        if c == '\\' then
            escaped = true
        elseif c == ';' and not escaped then
            val = convert_value and convert_value(val) or val
            table.insert(array, val)
            val = ""
        else
            val = val .. tostring(c)
            escaped = false
        end
    end
    return array
end

-- This is based on saveRouteProps() from 0.5.
-- Depends on the 0.5 runtime, obviously.
function saveRouteProps(source)
    local dest = {}
    for key,props in pairs(source) do
        dest[key] = Json.Object {
          volume = props.volume,
          mute = props.mute,
          channelVolumes = props.channelVolumes and Json.Array (props.channelVolumes),
          channelMap = props.channelMap and Json.Array (props.channelMap),
          latencyOffsetNsec = props.latencyOffsetNsec,
          iec958Codecs = props.iec958Codecs and Json.Array (props.iec958Codecs),
        }:to_string()
    end

    state:save(dest)
end

function tbltostring(t)
    if type(t) == "table" then
        local rs = '['
        for a, b in pairs(t) do
            rs = rs .. string.format(' %s: %s, ', a, tbltostring(b))
        end
        return rs .. ']'
    else
        return t
    end
end

print('[     ] Parsing 0.4-compatible routes.')
local joined_keys = {}
for i, v in pairs(state_t) do
    local k
    local prop
    local has_route_prop = false
    prop = i:gsub("^([^:]+):([^:]+):([^:]+)", function(c, direction, name)
        if (not c) or (not name)
            or (direction ~= 'input' and direction ~= 'output')
            then
            return false
        end
        has_route_prop = true
        k = string.format("%s:%s:%s", c, direction, name)
        return ""
    end)
    if has_route_prop then
        joined_keys[k] = joined_keys[k] or {}
        storeprop(joined_keys[k], prop, v)
    end
end

for k, j in pairs(joined_keys) do
    print(string.format('[trace] "%s":', k))
    for k, v in pairs(j) do
        print('[trace]', k, tbltostring(v))
    end
end

if has_old_props then
    print('[     ] Saving stored routes.')
    saveRouteProps(joined_keys)
end

print('[     ] Done.')
